@@ -1,46 +1,3 @@
-##### Functions for spatial lag regimes model####
-#' Estimation of spatial regimes models
-#' @name lag_regimes
-#' @param formula a symbolic description of the model of the form \code{y ~ x_f | x_v | wx | h_f | h_v | wh} where \code{y} is the dependent variable, \code{x_f} are the regressors that do not vary by regimes,  \code{x_v} are the regressors that vary by regimes, \code{wx} are the spatially lagged regressors, \code{h_f} are the instruments that do not vary by regimes,  \code{h_v} are the instruments that vary by regimes, \code{wh} are the spatially lagged instruments.
-#' @param data the data of class \code{data.frame}.
-#' @param listw a spatial weighting matrix of class \code{listw}, \code{matrix} or \code{Matrix}
-#' @param rgv an object of class \code{formula} to identify the regime variables
-#' @param wy_rg default \code{wy_rg = FALSE}, the lagged dependent variable does not vary by regime (see details)
-#' @param het heteroskedastic variance-covariance matrix
-#' @param cl record calls
-#' @param object an object of class lag_regime
-#' @param ... additional arguments
-#' @param x an object of class lag_regime
-#' @param digits number of digits
-#'
-#' @return an object of class \code{"lag_regimes"}. A \code{list} with six elements. The first element of the \code{list} contains the estimation results. The other elements are needed for printing.
-
-#' @examples
-#' data("natreg")
-#' data("ws_6")
-#' form <-  HR90  ~ 0 | MA90 + PS90 +
-#' RD90 + UE90 | 0 | 0 | MA90 + PS90 +
-#' RD90 + FH90 + FP89 + GI89 | 0
-#'
-#' form1 <-  HR90  ~ MA90 -1 |  PS90 +
-#' RD90 + UE90 | 0 | MA90 -1 |  PS90 +
-#' RD90 + FH90 + FP89 + GI89 | 0
-#'
-#' split  <- ~ REGIONS
-#'
-#' ###############################
-#' #  Spatial Lag regimes model  #
-#' ###############################
-#' mod4 <- spregimes(formula = form, data = natreg,
-#' rgv = split, listw = ws_6, model = "lag",
-#' het = TRUE, wy_rg = TRUE)
-#' summary(mod4)
-#' mod5 <- spregimes(formula = form1, data = natreg,
-#' rgv = split, listw = ws_6, model = "lag",
-#' het = TRUE, wy_rg = TRUE)
-#' summary(mod5)
-#'
-
 
 lag_regimes <- function(formula, data, listw, rgv,
                    het, cl, wy_rg){
@@ -61,164 +18,11 @@ lag_regimes <- function(formula, data, listw, rgv,
   nameswx  <- intro[[9]]
   res <- spatial.ivreg.regimes(as.matrix(y), as.matrix(Zmat), as.matrix(Hmat), het)
   res <- list(res, cl, colnames.end,  colnames.instr, colinstr, nameswx)
-  class(res) <- "lag_regimes"
+  class(res) <- c("spregimes","lag_regimes")
   return(res)
 
 }
 
-### S3 methods ----
-
-#' @rdname lag_regimes
-#' @method coef lag_regimes
-#' @export
-coef.lag_regimes <- function(object, ...){
-  object[[1]][[1]]
-}
-
-
-#' @rdname lag_regimes
-#' @method vcov lag_regimes
-#' @import stats
-#' @export
-vcov.lag_regimes <- function(object, ...){
-  V <- object[[1]][[2]]
-  return(V)
-}
-
-
-#' @rdname lag_regimes
-#' @method print lag_regimes
-#' @import stats
-#' @export
-print.lag_regimes <- function(x,
-                            digits = max(3, getOption("digits") - 3),
-                            ...)
-{
-  cat("Call:\n")
-  print(x[[2]])
-
-  cat("\nCoefficients:\n")
-
-  print.default(format(drop(coef(x)), digits = digits), print.gap = 2,
-                quote = FALSE)
-  cat("\n")
-  invisible(x)
-}
-
-
-
-#' @rdname lag_regimes
-#' @method summary lag_regimes
-#' @import stats
-#' @export
-summary.lag_regimes <- function(object, ...){
-  b                   <- coef(object)
-  std.err             <- sqrt(diag(vcov(object)))
-  z                   <- b / std.err
-  p                   <- 2 * (1 - pnorm(abs(z)))
-  CoefTable           <- cbind(b, std.err, z, p)
-  colnames(CoefTable) <- c("Estimate", "Std. Error", "z-value", "Pr(>|z|)")
-  object$CoefTable    <- CoefTable
-  class(object)       <- c("summary.lag_regimes", "lag_regimes")
-  return(object)
-}
-
-
-#' @rdname lag_regimes
-#' @method print summary.lag_regimes
-#' @import stats
-#' @export
-print.summary.lag_regimes <- function(x,
-                                    digits = max(5, getOption("digits") - 3),
-                                    ...)
-{
-  if(is.null(x[[5]])){
-
-    if(is.null(x[[6]])){
-  cat("        ------------------------------------------------------------\n")
-  cat("                          Spatial Lag Regimes Model \n")
-  cat("        ------------------------------------------------------------\n")
-  cat("\nCall:\n")
-  cat(paste(deparse(x[[2]]), sep = "\n", collapse = "\n"), "\n\n", sep = "")
-
-  cat("\nCoefficients:\n")
-  printCoefmat(x$CoefTable, digits = digits, P.values = TRUE, has.Pvalue = TRUE)
-
-  cat("\nEndogenous variables:\n")
-
-  cat(paste(unlist(x[[3]]), sep=" "))
-
-  cat("\nInstruments:\n")
-
-  cat(paste(x[[4]], sep=" "))
-    }
-    else{
-
-      cat("        ------------------------------------------------------------\n")
-      cat("                          Spatial Durbin Regimes Model \n")
-      cat("        ------------------------------------------------------------\n")
-      cat("\nCall:\n")
-      cat(paste(deparse(x[[2]]), sep = "\n", collapse = "\n"), "\n\n", sep = "")
-
-      cat("\nCoefficients:\n")
-      printCoefmat(x$CoefTable, digits = digits, P.values = TRUE, has.Pvalue = TRUE)
-
-      cat("\nEndogenous variables:\n")
-
-      cat(paste(unlist(x[[3]]), sep=" "))
-
-      cat("\nInstruments:\n")
-
-      cat(paste(x[[4]], sep=" "))
-    }
-
-  }
-  else{
-
-    if(is.null(x[[6]])){
-    cat("        ------------------------------------------------------------\n")
-    cat("                          Spatial Lag Regimes Model \n")
-    cat("                     with additional endogenous variables \n")
-    cat("        ------------------------------------------------------------\n")
-    cat("\nCall:\n")
-    cat(paste(deparse(x[[2]]), sep = "\n", collapse = "\n"), "\n\n", sep = "")
-
-    cat("\nCoefficients:\n")
-    printCoefmat(x$CoefTable, digits = digits, P.values = TRUE, has.Pvalue = TRUE)
-
-    cat("\nEndogenous variables:\n")
-
-    cat(paste(unlist(x[[3]]), sep=" "))
-
-    cat("\nInstruments:\n")
-
-    cat(paste(x[[4]], sep=" "))
-    }
-    else{
-      cat("        ------------------------------------------------------------\n")
-      cat("                          Spatial Durbin Regimes Model \n")
-      cat("                     with additional endogenous variables \n")
-      cat("        ------------------------------------------------------------\n")
-      cat("\nCall:\n")
-      cat(paste(deparse(x[[2]]), sep = "\n", collapse = "\n"), "\n\n", sep = "")
-
-      cat("\nCoefficients:\n")
-      printCoefmat(x$CoefTable, digits = digits, P.values = TRUE, has.Pvalue = TRUE)
-
-      cat("\nEndogenous variables:\n")
-
-      cat(paste(unlist(x[[3]]), sep=" "))
-
-      cat("\nInstruments:\n")
-
-      cat(paste(x[[4]], sep=" "))
-    }
-
-  }
-  invisible(x)
-}
-
-
 
 
 
@@ -598,8 +402,11 @@ spatial.ivreg.regimes <-function(y, Zmat, Hmat, het){
   Zpy <- crossprod(Zp,y)
   delta <- crossprod(ZpZpi,Zpy)
   yp <- Zmat %*% delta
+  yp <- array(yp, dim = c(length(yp),1),
+              dimnames = list(seq(1, length(yp)), ""))
   e <- y - yp
-
+  e <- array(e, dim = c(length(e),1),
+             dimnames = list(seq(1, length(e)), ""))
     if(het)	{
 
       s2 <- crossprod(e) /df
@@ -613,7 +420,9 @@ spatial.ivreg.regimes <-function(y, Zmat, Hmat, het){
       vardelta <- ZpZpi * as.numeric(s2)
     }
 
-  result <- list(coefficients = delta, var = vardelta, residuals = e)
+  result <- list(coefficients = delta, var = vardelta,
+                 residuals = e,
+                 X = Zmat, y = y, yp = yp)
   return(result)
 }
 
